import { AutoMap } from '@automapper/classes';
import { ArrayMinSize, IsArray, IsEnum, ValidateNested } from 'class-validator';
import { PointType } from '../../../geography/domain/types/point-type.enum';
import { Point } from '../../../geography/domain/types/point.type';

export class Ad {
  @AutoMap()
  uuid: string;

  @AutoMap()
  driver: boolean;

  @AutoMap()
  passenger: boolean;

  @AutoMap()
  frequency: number;

  @AutoMap()
  fromDate: Date;

  @AutoMap()
  toDate: Date;

  @AutoMap()
  monTime: string;

  @AutoMap()
  tueTime: string;

  @AutoMap()
  wedTime: string;

  @AutoMap()
  thuTime: string;

  @AutoMap()
  friTime: string;

  @AutoMap()
  satTime: string;

  @AutoMap()
  sunTime: string;

  @AutoMap()
  monMargin: number;

  @AutoMap()
  tueMargin: number;

  @AutoMap()
  wedMargin: number;

  @AutoMap()
  thuMargin: number;

  @AutoMap()
  friMargin: number;

  @AutoMap()
  satMargin: number;

  @AutoMap()
  sunMargin: number;

  @AutoMap()
  driverDuration: number;

  @AutoMap()
  driverDistance: number;

  @AutoMap()
  passengerDuration: number;

  @AutoMap()
  passengerDistance: number;

  @IsEnum(PointType)
  @AutoMap()
  originType: PointType;

  @IsEnum(PointType)
  @AutoMap()
  destinationType: PointType;

  @IsArray()
  @ArrayMinSize(2)
  @ValidateNested({ each: true })
  @AutoMap()
  waypoints: Array<Point>;

  @AutoMap()
  direction: string;

  @AutoMap()
  fwdAzimuth: number;

  @AutoMap()
  backAzimuth: number;

  @AutoMap()
  seatsDriver: number;

  @AutoMap()
  seatsPassenger: number;

  @AutoMap()
  seatsUsed: number;

  @AutoMap()
  createdAt: Date;

  @AutoMap()
  updatedAt: Date;
}
