import { Injectable } from '@nestjs/common';
import { ICreateGeorouter } from '../../domain/interfaces/georouter-creator.interface';
import { IGeorouter } from '../../domain/interfaces/georouter.interface';
import { GraphhopperGeorouter } from './graphhopper-georouter';
import { HttpService } from '@nestjs/axios';
import { MatcherGeodesic } from './geodesic';
import {
  MatcherException,
  MatcherExceptionCode,
} from '../../exceptions/matcher.exception';

@Injectable()
export class GeorouterCreator implements ICreateGeorouter {
  constructor(
    private readonly httpService: HttpService,
    private readonly geodesic: MatcherGeodesic,
  ) {}

  create = (type: string, url: string): IGeorouter => {
    switch (type) {
      case 'graphhopper':
        return new GraphhopperGeorouter(url, this.httpService, this.geodesic);
      default:
        throw new MatcherException(
          MatcherExceptionCode.INVALID_ARGUMENT,
          'Unknown geocoder',
        );
    }
  };
}
